# == Schema Information
#
# Table name: reviews
#
#  id              :integer          not null, primary key
#  name            :string
#  description     :text
#  pros            :text
#  cons            :text
#  reviewable_id   :integer
#  reviewable_type :string
#  user_id         :integer
#  created_at      :datetime         not null
#  updated_at      :datetime         not null
#  rates_count     :integer          default("0")
#

class Review < ApplicationRecord
  # *current_user* review *reviewable*
  include Trackable
  include Complainable

  after_save :calculate_review_rate

  belongs_to :reviewable, polymorphic: true
  belongs_to :user

  has_many :review_ratings
  has_many :ratings, through: :review_ratings
  has_many :rates

  validates_presence_of :user, :reviewable
  validates_inclusion_of :reviewable_type, in: %w(LearningObject Collection), message: 'Only LearningObjects and Collections are reviewable.'
  validates_uniqueness_of :user, scope: [:reviewable_id, :reviewable_type]

  accepts_nested_attributes_for :review_ratings

  acts_as_paranoid
  has_paper_trail

  def rating_values
    review_ratings.includes(:rating).map { |r| { rating: r.rating, value: r.value } }
  end

  def rating_average
    return 0.0 if review_ratings.empty?
    review_ratings.average(:value).floor(1).to_f
  end

  def rates_percentage
    return 0.0 if rates.count < 5
    approved = rates.where(approves: true).count
    (approved.to_f / rates.count.to_f) * 100
  end

  def rated?(user)
    !Rate.find_by(user: user, review: self).nil?
  end

  def user_approves?(user)
    return Rate.find_by(user: user, review: self).approves if rated? user
    nil
  end

  def recipient
    reviewable
  end

  private

  def calculate_review_rate
    ReviewAverageCalculatorWorker.perform_async(reviewable.id, reviewable.class.name)
  end

end
